#include "stph.h"
#include <stdio.h>

/* for select(): */
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

char grab_input(int ms_wait);


void add_task_cb(const struct StphTask *t, char *data)
{
  printf("Task says %s: xwin: %ld, class: `%-.20s', instance: `%-.20s'\n",
         data, t->xwin, t->class, t->instance);
  printf("   name: `%-.25s', icon name: `%-.25s'\n", t->name, t->icon_name);
  printf("   icon pixmap: %ld, icon mask: %ld, icon window: %ld\n",
         t->icon_pixmap, t->icon_mask, t->icon_window);
  printf("   kwm mini-icon pixmap: %ld, mask: %ld\n",
         t->kwm_icon_pixmap, t->kwm_icon_mask);
}

void remove_task_cb(const struct StphTask *t)
{
  printf("Task removed: %ld (%-.30s)\n", t->xwin, t->name);
}

void task_name_cb(const struct StphTask *t)
{
  printf("Task name changed: %ld (%-.30s)\n", t->xwin, t->name);
}

void task_state_cb(const struct StphTask *t)
{
  printf("Task state changed: %ld (%-.30s)\n", t->xwin, t->name);
  printf("  New state:");

  if (t->state == 0) {
          printf(" [no state bits set now]");
  } else {
          if (t->state & STPH_STATE_STICKY)
                  printf(" sticky");
          if (t->state & STPH_STATE_MINIMIZED)
                  printf(" minimized");
          if (t->state & STPH_STATE_MAXIMIZED_VERT)
                  printf(" vert-max'd");
          if (t->state & STPH_STATE_MAXIMIZED_HORIZ)
                  printf(" horiz-max'd");
          if (t->state & STPH_STATE_HIDDEN)
                  printf(" hidden");
          if (t->state & STPH_STATE_SHADED)
                  printf(" shaded");
          if (t->state & STPH_STATE_HID_WORKSPACE)
                  printf(" hid-workspace");
          if (t->state & STPH_STATE_HID_TRANSIENT)
                  printf(" hid-transient");
          if (t->state & STPH_STATE_FIXED_POSITION)
                  printf(" fixedpos");
          if (t->state & STPH_STATE_ARRANGE_IGNORE)
                  printf(" arrange-ignore");
  }
  printf("\n");
}

void task_hints_cb(const struct StphTask *t)
{
  printf("Task hints changed: %ld (%-.30s)\n", t->xwin, t->name);
  printf("  New hints:");

  if (t->hints == 0) {
          printf(" (no hints set now)");
  } else {
          if (t->hints & STPH_HINTS_SKIP_FOCUS)
                  printf(" skip-focus");
          if (t->hints & STPH_HINTS_SKIP_WINLIST)
                  printf(" skip-winlist");
          if (t->hints & STPH_HINTS_SKIP_TASKBAR)
                  printf(" skip-taskbar");
          if (t->hints & STPH_HINTS_GROUP_TRANSIENT)
                  printf(" group-transient");
          if (t->hints & STPH_HINTS_FOCUS_ON_CLICK)
                  printf(" focus-on-click");
          if (t->hints & STPH_HINTS_DO_NOT_COVER)
                  printf(" do-not-cover");
  }
  printf("\n");
}

void task_focus_cb(const struct StphTask *t)
{
  printf("Task %s: %ld (%-.30s)\n", t->focused ? "focused" : "unfocused",
         t->xwin, t->name);
}

void task_iconify_cb(const struct StphTask *t)
{
  printf("Task %s: %ld (%-.30s)\n", t->iconified ? "iconified" : "un-iconified",
         t->xwin, t->name);
}

void task_frame_cb(const struct StphTask *t)
{
  printf("Task frame now %ld: %ld (%-.30s)\n", t->xframe, t->xwin, t->name);
}

void task_frame_dims_cb(const struct StphTask *t)
{
  printf("Task frame dims now %d,%d: %ld (%-.30s)\n", t->frame_width, t->frame_height,
         t->xwin, t->name);
}

void task_frame_pos_cb(const struct StphTask *t)
{
  printf("Task frame pos now %d,%d: %ld (%-.30s)\n", t->frame_x, t->frame_y, t->xwin, t->name);
}

void task_win_dims_cb(const struct StphTask *t)
{
  printf("Task win dims now %d,%d: %ld (%-.30s)\n", t->win_width, t->win_height, t->xwin, t->name);
}

void task_win_pos_cb(const struct StphTask *t)
{
  printf("Task win pos now %d,%d: %ld (%-.30s)\n", t->win_x, t->win_y, t->xwin, t->name);
}

void task_layer_cb(const struct StphTask *t)
{
  printf("Task layer changed to %d: %ld (%-.30s)\n", t->layer, t->xwin, t->name);
}

void task_desktop_cb(const struct StphTask *t)
{
  printf("Task moved to workspace %d: %ld (%-.30s)\n", t->desktop, t->xwin, t->name);
  /* "desktop" == "workspace" */
}

void task_area_cb(const struct StphTask *t)
{
  printf("Task moved to viewport %d,%d: %ld (%-.30s)\n", t->harea, t->varea, t->xwin, t->name);
  /* "area" == "viewport" */
}

void task_icon_pixmap_cb(const struct StphTask *t)
{
  printf("Task icon pixmap/mask changed: pixmap: %ld mask: %ld\n",
         t->icon_pixmap, t->icon_mask);
}

void task_icon_window_cb(const struct StphTask *t)
{
  printf("Task icon window changed: now %ld\n", t->icon_window);
}

void task_kwm_icon_cb(const struct StphTask *t)
{
  printf("KWM mini-icon changed: pixmap: %ld, mask: %ld\n",
         t->kwm_icon_pixmap, t->kwm_icon_mask);
}

void area_cb(struct StphRoot *r)
{
  printf("Viewport changed: %d,%d\n", r->harea, r->varea);
}

void area_count_cb(struct StphRoot *r)
{
  printf("Number of viewports changed: now %dx%d\n", r->harea_count, r->varea_count);
}

void workspace_cb(struct StphRoot *r)
{
  printf("Workspace changed: %d\n", r->workspace);
}

void workspace_count_cb(struct StphRoot *r)
{
  printf("Number of workspaces changed: now %d\n", r->workspace_count);
}


char grab_input(int ms_wait)
{
  fd_set set;
  struct timeval timeout;
  char ret;
  
  FD_ZERO(&set);
  FD_SET(STDIN_FILENO, &set);
  timeout.tv_sec = 0;
  timeout.tv_usec = ms_wait*1000;

  if (select(FD_SETSIZE, &set, NULL, NULL, &timeout)) {
          ret = getchar();
          return ret; 
  } else {
          return 'z';
  }
  
}


int main(int argc, char **argv)
{
  char ch;
  struct StphRoot *rprops;

  char *datastr = "Hello!";
  
  stph_connect(STPH_ADD_TASK_CB, add_task_cb, datastr);
  stph_connect(STPH_REMOVE_TASK_CB, remove_task_cb, NULL);
  stph_connect(STPH_TASK_NAME_CB, task_name_cb, NULL);
  stph_connect(STPH_TASK_STATE_CB, task_state_cb, NULL);
  stph_connect(STPH_TASK_HINTS_CB, task_hints_cb, NULL);
  stph_connect(STPH_TASK_FOCUS_CB, task_focus_cb, NULL);
  stph_connect(STPH_TASK_ICONIFY_CB, task_iconify_cb, NULL);
  stph_connect(STPH_TASK_FRAME_CB, task_frame_cb, NULL);
  stph_connect(STPH_TASK_FRAME_DIMS_CB, task_frame_dims_cb, NULL);
  stph_connect(STPH_TASK_FRAME_POS_CB, task_frame_pos_cb, NULL);
  stph_connect(STPH_TASK_WIN_DIMS_CB, task_win_dims_cb, NULL);
  stph_connect(STPH_TASK_WIN_POS_CB, task_win_pos_cb, NULL);
  stph_connect(STPH_TASK_LAYER_CB, task_layer_cb, NULL);
  stph_connect(STPH_TASK_DESKTOP_CB, task_desktop_cb, NULL);
  stph_connect(STPH_TASK_AREA_CB, task_area_cb, NULL);
  stph_connect(STPH_TASK_ICON_PIXMAP_CB, task_icon_pixmap_cb, NULL);
  stph_connect(STPH_TASK_ICON_WINDOW_CB, task_icon_window_cb, NULL);
  stph_connect(STPH_TASK_KWM_ICON_CB, task_kwm_icon_cb, NULL);
  stph_connect(STPH_AREA_CB, area_cb, NULL);
  stph_connect(STPH_AREA_COUNT_CB, area_count_cb, NULL);
  stph_connect(STPH_WORKSPACE_CB, workspace_cb, NULL);
  stph_connect(STPH_WORKSPACE_COUNT_CB, workspace_count_cb, NULL);
  
  
  stph_initialise();

  rprops = stph_get_root_props();
  printf("========================================================\n");
  printf("Current workspace is %d\n", rprops->workspace);
  printf("Current viewport is %d,%d\n", rprops->harea, rprops->varea);
  printf("There are %d workspace(s)\n", rprops->workspace_count);
  printf("There are %d horizontal and %d vertical viewport(s) in this workspace\n",
         rprops->harea_count, rprops->varea_count);
  printf("========================================================\n");
  
  
  
  while (1) {
          stph_mainiteration();

          ch = grab_input(50); /* pause 50 ms */
          if (ch == 'q' || ch == 'Q')
                  break;
  }

  return 0;
}

