/* stph.h -- Sawmill-inspired Taskbar and Pager Helper

   Copyright (C) 2000 Rob Hodges <s323140@student.uq.edu.au>

   This file is part of stph.

   stph is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   stph is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with stph; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */


#ifndef __STPH_H__
#define __STPH_H__ 1

#include <X11/Xlib.h>

/* --- defines --- */

/* task-related callbacks */
#define STPH_ADD_TASK_CB 	1
#define STPH_REMOVE_TASK_CB 	2
#define STPH_TASK_NAME_CB 	3
#define STPH_TASK_ICON_NAME_CB 	4
#define STPH_TASK_CLASS_CB	5 /* covers both class and instance attrs */
#define STPH_TASK_STATE_CB 	6
#define STPH_TASK_HINTS_CB 	7
#define STPH_TASK_FOCUS_CB 	8
#define STPH_TASK_ICONIFY_CB 	9
#define STPH_TASK_FRAME_CB 	10
#define STPH_TASK_FRAME_DIMS_CB	11
#define STPH_TASK_FRAME_POS_CB	12
#define STPH_TASK_WIN_DIMS_CB 	13
#define STPH_TASK_WIN_POS_CB	14
#define STPH_TASK_LAYER_CB 	15
#define STPH_TASK_DESKTOP_CB 	16
#define STPH_TASK_AREA_CB 	17
#define STPH_TASK_ICON_PIXMAP_CB 18
#define STPH_TASK_ICON_WINDOW_CB 19
#define STPH_TASK_KWM_ICON_CB   20

/* this is for internal use: */    /****** N.B.!!! change this if callbacks added! ******/
#define STPH_MAX_TASK_CALLBACK	20 /****** N.B.!!! change this if callbacks added! ******/
                                   /****** N.B.!!! change this if callbacks added! ******/

/* workspace/viewport callbacks: */
#define STPH_AREA_CB 		21
#define STPH_AREA_COUNT_CB 	22
#define STPH_WORKSPACE_CB 	23
#define STPH_WORKSPACE_COUNT_CB 24

/* this is for internal use: */    /****** N.B.!!! change this if callbacks added! ******/
#define STPH_MAX_ROOT_CALLBACK	24 /****** N.B.!!! change this if callbacks added! ******/
				   /****** N.B.!!! change this if callbacks added! ******/
/* --- enums --- */

typedef enum
{
        STPH_LAYER_DESKTOP     = 0,
        STPH_LAYER_BELOW       = 2,
        STPH_LAYER_NORMAL      = 4,
        STPH_LAYER_ONTOP       = 6,
        STPH_LAYER_DOCK        = 8,
        STPH_LAYER_ABOVE_DOCK  = 10
} StphLayer;

typedef enum
{
        STPH_STATE_STICKY          = (1<<0), /* window appears on all desktops+areas */
        STPH_STATE_MINIMIZED       = (1<<1), /* ??? */
        STPH_STATE_MAXIMIZED_VERT  = (1<<2), /* window vertically maximized */
        STPH_STATE_MAXIMIZED_HORIZ = (1<<3), /* window horizontally maximized */
        STPH_STATE_HIDDEN          = (1<<4), /* ??? */
        STPH_STATE_SHADED          = (1<<5), /* only show window's wm decorations */
        STPH_STATE_HID_WORKSPACE   = (1<<6), /* ??? */
        STPH_STATE_HID_TRANSIENT   = (1<<7), /* owner of transient is hidden */
        STPH_STATE_FIXED_POSITION  = (1<<8), /* don't scroll with area */
        STPH_STATE_ARRANGE_IGNORE  = (1<<9)  /* ignore for auto arranging */
} StphState;

typedef enum
{
        STPH_HINTS_SKIP_FOCUS      = (1<<0), /* no ALT+Tab focussing */
        STPH_HINTS_SKIP_WINLIST    = (1<<1), /* not in win list */
        STPH_HINTS_SKIP_TASKBAR    = (1<<2), /* not on taskbar */
        STPH_HINTS_GROUP_TRANSIENT = (1<<3), /* ??? */
        STPH_HINTS_FOCUS_ON_CLICK  = (1<<4), /* app only accepts focus when clicked */
        STPH_HINTS_DO_NOT_COVER    = (1<<5)  /* attempt to not cover this window */
} StphHints;

typedef enum
{
        STPH_WIN_LAYER_PCOL 		= (1<<0),
        STPH_WIN_STATE_PCOL 		= (1<<1),
        STPH_WIN_HINTS_PCOL 		= (1<<2),
        STPH_WIN_APP_STATE_PCOL		= (1<<3),
        STPH_WIN_EXPANDED_SIZE_PCOL 	= (1<<4),
        STPH_WIN_ICONS_PCOL 		= (1<<5),
        STPH_WIN_WORKSPACE_PCOL 	= (1<<6),
        STPH_WIN_WORKSPACE_COUNT_PCOL 	= (1<<7),
        STPH_WIN_WORKSPACE_NAMES_PCOL 	= (1<<8),
        STPH_WIN_CLIENT_LIST_PCOL 	= (1<<9)
} StphProtocols;


/* --- structs --- */

struct StphRoot {
        int compliant_wm;
        StphProtocols supported_protocols;

        int harea, varea;
        int harea_count, varea_count;

        int workspace;
        int workspace_count;

        /* FIXME: workspace names not yet supported */
        /* (Sawmill doesn't support them, so this is a low priority... */
};


struct StphTask {
        /* window state & hints */
        char *name;      /* eg. "Emacs: *scratch*" */
        char *icon_name; /* eg. "*scratch*" */
        char *class;     /* eg. "Netscape" */
                         /* (this is task.wm_class in python since 'class' is a keyword in python) */
        char *instance;  /* eg. "Composer" or "Navigator" -- IOW this is the subclass -- see ICCCM */
        StphState state;
        StphHints hints;
        int focused;
        int iconified;

        Pixmap icon_pixmap;
        Pixmap icon_mask;
        Window icon_window;
        Pixmap kwm_icon_pixmap;
        Pixmap kwm_icon_mask;
        
        /* geometry */
        int frame_x, frame_y;
        int frame_width, frame_height;
        int win_x, win_y;
        int win_width, win_height;

        /* desktop; desktop area; layer */
        int desktop; /* == workspace */
        int harea, varea; /* == viewport */
        int layer;
        
        /* windows */
        Window xwin;
        Window xframe;
};

/* --- prototypes --- */

/* Initialisation; Event checker: */
int stph_initialise(void);
void stph_mainiteration(void);

/* Set up callbacks: */
void stph_connect(int cb_type, void (*func)(), void *data);

/* Convenience functions: (these all return 0 on success; nonzero on failure */
int stph_focus_task(struct StphTask *t);
int stph_iconify_task(struct StphTask *t);
int stph_uniconify_task(struct StphTask *t);
int stph_iconify_task_toggle(struct StphTask *t);
int stph_close_task(struct StphTask *t); /* preferable to kill functions in most circumstances */
int stph_kill_task(struct StphTask *t); /* use only at user's direct request! */
int stph_close_or_kill_task(struct StphTask *t); /* dangerous for "wanna save first?" apps! use
                                                     only at user's direct request! */
int stph_move_task(struct StphTask *t, int x, int y);
int stph_set_current_desktop(long desktop);
int stph_set_current_area(long harea, long varea);

int stph_raise_task_window(struct StphTask *t); /* raises task to top of stacking order */

int stph_shade_task(struct StphTask *t);
int stph_unshade_task(struct StphTask *t);
int stph_shade_task_toggle(struct StphTask *t);


/* Explicit access to stph structures: (mostly useful for debugging) */
struct StphTask *stph_task_from_xwin(Window xwin); /* returns NULL if task not found */
struct StphRoot *stph_get_root_props(void);

#endif /* ndef __STPH_H__ */
